Package: methylCC
Title: Estimate the cell composition of whole blood in DNA methylation
        samples
Version: 1.24.0
Authors@R: c(
    person(c("Stephanie", "C."), "Hicks", role = c("aut", "cre"), 
		email = "shicks19@jhu.edu", 
		comment = c(ORCID = "0000-0002-7858-0231")),
    person("Rafael", "Irizarry", role = "aut", 
		comment = c(ORCID = "0000-0002-3944-4309")))
Imports: Biobase, GenomicRanges, IRanges, S4Vectors, dplyr, magrittr,
        minfi, bsseq, quadprog, plyranges, stats, utils, bumphunter,
        genefilter, methods, IlluminaHumanMethylation450kmanifest,
        IlluminaHumanMethylation450kanno.ilmn12.hg19
Depends: R (>= 3.6), FlowSorted.Blood.450k
Suggests: rmarkdown, knitr, testthat (>= 2.1.0), BiocGenerics,
        BiocStyle, tidyr, ggplot2
Description: A tool to estimate the cell composition
    of DNA methylation whole blood sample measured on any
    platform technology (microarray and sequencing).
biocViews: Microarray, Sequencing, DNAMethylation, MethylationArray,
        MethylSeq, WholeGenome
VignetteBuilder: knitr
RoxygenNote: 6.1.1
Encoding: UTF-8
License: GPL-3
BugReports: https://github.com/stephaniehicks/methylCC/
URL: https://github.com/stephaniehicks/methylCC/
git_url: https://git.bioconductor.org/packages/methylCC
git_branch: RELEASE_3_22
git_last_commit: 98b2564
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
NeedsCompilation: no
Packaged: 2025-10-30 05:02:06 UTC; biocbuild
Author: Stephanie C. Hicks [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-7858-0231>),
  Rafael Irizarry [aut] (ORCID: <https://orcid.org/0000-0002-3944-4309>)
Maintainer: Stephanie C. Hicks <shicks19@jhu.edu>
