# Generated by roxygen2: do not edit by hand

export(estimatecc)
exportClasses(estimatecc)
exportMethods(cell_counts)
import(FlowSorted.Blood.450k)
import(GenomicRanges)
import(IlluminaHumanMethylation450kanno.ilmn12.hg19)
import(IlluminaHumanMethylation450kmanifest)
import(methods)
import(minfi)
importFrom(Biobase,pData)
importFrom(Biobase,sampleNames)
importFrom(IRanges,subsetByOverlaps)
importFrom(S4Vectors,queryHits)
importFrom(S4Vectors,subjectHits)
importFrom(bsseq,BSseq)
importFrom(bsseq,getBSseq)
importFrom(bumphunter,bumphunter)
importFrom(bumphunter,clusterMaker)
importFrom(bumphunter,loessByCluster)
importFrom(dplyr,funs)
importFrom(dplyr,group_by)
importFrom(dplyr,summarise_all)
importFrom(genefilter,rowttests)
importFrom(magrittr,"%>%")
importFrom(plyranges,arrange)
importFrom(quadprog,solve.QP)
importFrom(stats,model.matrix)
importFrom(stats,runif)
importFrom(stats,var)
importFrom(utils,data)
importFrom(utils,head)
