#' @name FlowSorted.Blood.450k.sub 
#' @title A reduced size of the FlowSorted.Blood.450k dataset 
#' 
#' @description A reduced size of the 
#' \code{FlowSorted.Blood.450k} dataset 
#' 
#' The object was created using the script in /inst and
#' located in the /data folder.
#' 
#' @docType data 
#' @format A RGset object with 2e5 rows (probes)
#' and 6 columns (whole blood samples).
#' 
#' @rdname FlowSorted.Blood.450k.sub
#' 
NULL 

#' @name onMethRegions
#' @title Methylated regions for all celltypes
#'
#' @description
#' This is the script used to create the
#' \code{onMethRegions} data set.
#' The purpose is use in the \code{estimate_cc()}
#' function.
#'
#' The object was created using the script in /inst and
#' located in the /data folder.
#'
#' @docType data
#' @format add more here.
#'
#' @rdname onMethRegions
#'
NULL

#' @name offMethRegions
#' @title Unmethylated regions for all celltypes
#'
#' @description
#' This is the script used to create the
#' \code{offMethRegions} data set.
#' The purpose is use in the \code{estimate_cc()}
#' function.
#'
#' The object was created using the script in /inst and
#' located in the /data folder.
#'
#' @docType data
#' @format add more here.
#'
#' @rdname offMethRegions
#'
NULL
